# -*- coding: utf-8 -*-
"""demo URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.9/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Add an import:  from blog import urls as blog_urls
    2. Add a URL to urlpatterns:  url(r'^blog/', include(blog_urls))
"""
from django.conf.urls import include, patterns, url
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),
]

urlpatterns += i18n_patterns(
    url(_(r'^'), include('store.urls', namespace='store')),
    url(_(r'^account/'), include('account.urls', namespace='account')),
)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings

# Default settings
BOILERPLATE_DEFAULTS = {
    'model_url_list_suffix': '_list',
    'app_url_index': 'home',
}

# Start with a copy of default settings
BOILERPLATE = BOILERPLATE_DEFAULTS.copy()

# Override with user settings from settings.py
BOILERPLATE.update(getattr(settings, 'BOILERPLATE', {}))


def get_boilerplate_setting(setting, default=None):
    """
    Read a setting
    """
    return BOILERPLATE.get(setting, default)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from django.core.files.base import ContentFile
from django.core.files.storage import default_storage as storage

from PIL import Image
import six


class ModelImageThumbs(object):
    IMAGESIZES = None

    def save(self, *args, **kwargs):
        response = super(ModelImageThumbs, self).save(*args, **kwargs)

        if self.image:
            filename = str(self.image.name)
            imgFile = Image.open(storage.open(filename))

            if imgFile.mode not in ('L', 'RGB'):
                imgFile = imgFile.convert('RGB')

            for field_name, size in six.iteritems(self.IMAGESIZES):
                field = getattr(self, field_name)

                if not field:
                    working = imgFile.copy()
                    working = working.resize(size, Image.ANTIALIAS)
                    fp = StringIO()
                    working.save(fp, 'JPEG', quality=95)
                    cf = ContentFile(fp.getvalue())
                    field.save(name=self.image.name, content=cf, save=True)
        else:
            for field_name, size in six.iteritems(self.IMAGESIZES):
                field = getattr(self, field_name)
                field.delete(save=True)

        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from . import views

urlpatterns = [
	url(_(r'^password/$'), views.ChangePassword.as_view(), name='password'),
	url(_(r'^login/$'), views.Login.as_view(), name='login'),
	url(_(r'^logout/$'), views.Logout.as_view(), name='logout'),
	url(_(r'^registration/$'), views.Registration.as_view(), name='registration'),
	url(_(r'^recover/$'), views.Recover.as_view(), name='recover'),
	url(_(r'^recover/confirm/(?P<uidb64>[0-9A-Za-z]+)-(?P<token>.+)/$'), views.RecoverConfirm.as_view(), name='recover_confirm'),
]


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from . import views

urlpatterns = [
	url(_(r'^$'), views.MovieList.as_view(), name='movie_list' ),
	url(_(r'^movies/add/$'), views.MovieCreate.as_view(), name='movie_create' ),
	# Actor
	url(_(r'^actors/$'), views.ActorList.as_view(), name='actor_list' ),
	url(_(r'^actors/add/$'), views.ActorCreate.as_view(), name='actor_create' ),
	url(_(r'^actors/(?P<pk>[0-9]+)/$'),views.ActorDetail.as_view(), name='actor_detail' ),
	url(_(r'^actors/(?P<pk>[0-9]+)/update/$'),views.ActorUpdate.as_view(), name='actor_update' ),
	url(_(r'^actors/(?P<pk>[0-9]+)/delete/$'),views.ActorDelete.as_view(), name='actor_delete' ),
]


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from django.views.generic import DetailView, ListView
from django.views.generic.edit import CreateView, UpdateView, DeleteView

from boilerplate.mixins import CreateMessageMixin, DeleteMessageMixin, ExtraFormsAndFormsetsMixin, ListActionsMixin, UpdateMessageMixin

from . import forms 
from .models import Actor, Movie

class ActorList(ListActionsMixin, ListView):
	action_list = (
		( _('Add'), 'create', 'primary', 'plus'),
	)
	model       = Actor
	paginate_by = 30

class ActorDetail(DetailView):
	model = Actor

class ActorCreate(ExtraFormsAndFormsetsMixin, CreateMessageMixin, CreateView):
	extra_form_list = (
		('contact_information', 'actor', forms.ContactForm),
	)
	form_class      = forms.ActorForm
	model           = Actor

class ActorUpdate(ExtraFormsAndFormsetsMixin, UpdateMessageMixin, UpdateView):
	extra_form_list = (
		('contact_information', 'actor', forms.ContactForm),
	)
	form_class      = forms.ActorForm
	model           = Actor

class ActorDelete(DeleteMessageMixin, DeleteView):
	model                = Actor
	success_url          = reverse_lazy('store:actor_list')
	template_name_suffix = '_form'

class MovieList(ListActionsMixin, ListView):
	action_list = (
		( _('Add'), 'create', 'primary', 'plus'),
	)
	model       = Movie
	paginate_by = 30

class MovieCreate(ExtraFormsAndFormsetsMixin, CreateMessageMixin, CreateView):
	form_class   = forms.MovieForm
	formset_list = (
		forms.MovieCastFormSet,
	)
	model        = Movie


# -*- coding: utf-8 -*-
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

from django import forms
from django.core.management import call_command
from django.contrib.messages import get_messages
from django.contrib.messages.storage import default_storage
from django.core.exceptions import PermissionDenied
from django.db.models import signals
from django.forms import inlineformset_factory
from django.contrib.admin.models import LogEntry
from django.contrib.auth.models import (
    AnonymousUser, ContentType, Permission, User
)
from django.test import RequestFactory, TestCase
try:
    from django.template import engines
except ImportError:
    from django.template.engines import Engine
from django.views.generic import DetailView, TemplateView
from django.views.generic.edit import CreateView, DeleteView, UpdateView

from .mail import SendEmail
from .mixins import (
    NoLoginRequiredMixin, ActionListMixin, UserCreateMixin, CreateMessageMixin,
    UpdateMessageMixin, DeleteMessageMixin, ExtraFormsAndFormsetsMixin,
    ParentCreateMixin, ParentSingleObjectMixin
)
from .signals import add_view_permissions


class TestModelForm(forms.ModelForm):
    class Meta:
        fields = '__all__'
        model = User


class TestForm(forms.Form):
    name = forms.CharField()


TestFormSet = inlineformset_factory(
    ContentType,
    Permission,
    fields='__all__',
    min_num=0
)


def render_template(text, context=None):
    """
    Create a template ``text`` that first loads boilerplate.
    """
    try:
        template = engines['django'].from_string(text)
    except Exception:
        template = Engine().from_string(text)
    if not context:
        context = {}
    return template.render(context)


def render_template_with_boilerplate(text, context=None):
    """
    Create a template ``text`` that first loads boilerplate.
    """
    if not context:
        context = {}
    return render_template("{% load boilerplate %}" + text, context)


def render_template_with_form(text, context=None):
    """
    Create a template ``text`` that first loads boilerplate.
    """
    if not context:
        context = {}
    if 'form' not in context:
        context['form'] = TestForm()
    return render_template_with_boilerplate(text, context)


def render_template_with_form_model(text, context=None):
    """
    Create a template ``text`` that first loads boilerplate.
    """
    if not context:
        context = {}
    if 'form' not in context:
        context['form'] = TestModelForm()
    return render_template_with_boilerplate(text, context)


def render_template_with_formset(text, context=None):
    """
    Create a template ``text`` that first loads boilerplate.
    """
    if not context:
        context = {}
    if 'form' not in context:
        context['formset'] = TestFormSet()
    return render_template_with_boilerplate(text, context)


def render_template_with_object(obj, text, context=None):
    """
    Create a template ``text`` that first loads boilerplate.
    """
    if not context:
        context = {}
    if 'object' not in context:
        context['object'] = obj
    return render_template_with_boilerplate(text, context)


def render_template_with_object_list(object_list, text, context=None):
    """
    Create a template ``text`` that first loads boilerplate.
    """
    if not context:
        context = {}
    if 'object_list' not in context:
        context['object_list'] = object_list
    return render_template_with_boilerplate(text, context)


class TemplateTest(TestCase):
    def setUp(self):
        self.form_model = TestModelForm()
        self.form = TestForm()
        self.o_user = User.objects.create(
            username="test",
            email="test@test.com"
        )
        self.qs_user = User.objects.all()

    def test_form_model_name(self):
        res = render_template_with_form_model(
            '{{ form|form_model_name }}'
        )
        self.assertEqual('user', res)

        res = render_template_with_form(
            '{{ form|form_model_name }}'
        )
        self.assertEqual('TestForm', res)

    def test_form_model_name_plural(self):
        res = render_template_with_form_model(
            '{{ form|form_model_name_plural }}'
        )
        self.assertEqual('users', res)

    def test_form_model_url(self):
        res = render_template_with_form_model(
            '{{ form|form_model_url }}'
        )
        self.assertEqual('auth:user_list', res)

    def test_form_app_name(self):
        res = render_template_with_form_model(
            '{{ form|form_app_name }}'
        )
        self.assertEqual('Authentication and Authorization', res)

    def test_form_app_url(self):
        res = render_template_with_form_model(
            '{{ form|form_app_url }}'
        )
        self.assertEqual('auth:home', res)

    def test_form_prefix(self):
        res = render_template_with_form_model(
            '{{ form|form_prefix }}'
        )
        self.assertEqual('testmodelform', res)

    def test_formset_model_name(self):
        res = render_template_with_formset(
            '{{ formset|formset_model_name }}'
        )
        self.assertEqual('permission', res)

    def test_formset_model_name_plural(self):
        res = render_template_with_formset(
            '{{ formset|formset_model_name_plural }}'
        )
        self.assertEqual('permissions', res)

    def test_model_name(self):
        res = render_template_with_object(
            self.o_user,
            '{{ object|model_name }}'
        )
        self.assertEqual('user', res)

    def test_model_name_plural(self):
        res = render_template_with_object(
            self.o_user,
            '{{ object|model_name_plural }}'
        )
        self.assertEqual('users', res)

    def test_model_app_name(self):
        res = render_template_with_object(
            self.o_user,
            '{{ object|model_app_name }}'
        )
        self.assertEqual('Authentication and Authorization', res)

    def test_model_app_url(self):
        res = render_template_with_object(
            self.o_user,
            '{{ object|model_app_url }}'
        )
        self.assertEqual('auth:home', res)

    def test_model_url(self):
        res = render_template_with_object(
            self.o_user,
            '{{ object|model_url }}'
        )
        self.assertEqual('auth:user_list', res)

    def test_queryset_app_name(self):
        res = render_template_with_object_list(
            self.qs_user,
            '{{ object_list|queryset_app_name }}'
        )
        self.assertEqual('Authentication and Authorization', res)

    def test_queryset_app_url(self):
        res = render_template_with_object_list(
            self.qs_user,
            '{{ object_list|queryset_app_url }}'
        )
        self.assertEqual('auth:home', res)

    def test_queryset_model_name_plural(self):
        res = render_template_with_object_list(
            self.qs_user,
            '{{ object_list|queryset_model_name_plural }}'
        )
        self.assertEqual('users', res)

    def test_queryset_model_url(self):
        res = render_template_with_object_list(
            self.qs_user,
            '{{ object_list|queryset_model_url }}'
        )
        self.assertEqual('auth:user_list', res)


class SignalTest(TestCase):
    def test_migration_output(self):
        signals.post_migrate.connect(add_view_permissions)
        out = StringIO()
        call_command('migrate', stdout=out)
        # self.assertIn('Added view permission', out.getvalue())


class NoLoginRequiredView(NoLoginRequiredMixin, TemplateView):
    template_name = 'any_template.html'


class ActionListView(ActionListMixin, TemplateView):
    action_list = (
        'Add', 'Change', 'Delete'
    )
    template_name = 'any_template.html'


class UserCreateView(UserCreateMixin, CreateView):
    fields = (
        'content_type', 'object_id', 'object_repr', 'action_flag',
    )
    model = LogEntry
    success_url = '/fake-path-success'

    def form_valid(self, form):
        super(UserCreateView, self).form_valid(form)
        return self.object


class CreateMessageView(CreateMessageMixin, CreateView):
    fields = ('username', 'email')
    model = User
    success_url = '/fake-path-success'

    def form_valid(self, form):
        super(CreateMessageView, self).form_valid(form)
        return self.render_to_response(self.get_context_data(form=form))


class UpdateMessageView(UpdateMessageMixin, UpdateView):
    fields = ('username', 'email')
    model = User
    success_url = '/fake-path-success'

    def get_object(self):
        obj, created = User.objects.get_or_create(username="gclooney")
        return obj

    def form_valid(self, form):
        super(UpdateMessageView, self).form_valid(form)
        return self.render_to_response(self.get_context_data(form=form))


class DeleteMessageView(DeleteMessageMixin, DeleteView):
    model = User
    success_url = '/fake-path-success'

    def get_object(self):
        obj, created = User.objects.get_or_create(username="gclooney")
        return obj

    def delete(self, request, *args, **kwargs):
        super(DeleteMessageView, self).delete(request, *args, **kwargs)
        return self.render_to_response({})


class ParentCreateView(ParentCreateMixin, CreateView):
    fields = ('codename', 'name')
    model = Permission
    parent_model = ContentType
    parent_relation_field = 'content_type'
    success_url = '/fake-path-success'

    def form_valid(self, form):
        super(ParentCreateView, self).form_valid(form)
        return self.object


class ParentSingleObjectView(ParentSingleObjectMixin, DetailView):
    model = Permission
    parent_model = ContentType
    parent_relation_field = 'content_type'


class ExtraFormsAndFormsetsView(ExtraFormsAndFormsetsMixin, UpdateView):
    PermissionFormSet = inlineformset_factory(
        ContentType,
        Permission,
        fields='__all__',
        extra=1,
        can_delete=False,

    )
    fields = '__all__'
    model = ContentType
    formset_list = (
        PermissionFormSet,
    )
    success_url = '/fake-path-success'

    def form_valid(self, form, extra_forms=None, formsets=None):
        super(ExtraFormsAndFormsetsView, self).form_valid(
            form, extra_forms, formsets
        )
        return self.object


class ExtraFormsAndFormsetsAndUpdateView(
    ExtraFormsAndFormsetsMixin, UpdateMessageMixin, UpdateView
):
    PermissionFormSet = inlineformset_factory(
        ContentType,
        Permission,
        fields='__all__',
        extra=1,
        can_delete=False,

    )
    fields = '__all__'
    model = ContentType
    formset_list = (
        PermissionFormSet,
    )
    success_url = '/fake-path-success'

    def form_valid(self, form, extra_forms=None, formsets=None):
        super(ExtraFormsAndFormsetsAndUpdateView, self).form_valid(
            form, extra_forms, formsets
        )
        return self.object


class MixinTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.user = User.objects.create_user(
            username='test',
            email='test@test.com'
        )

    def test_no_login_required_mixin_anonymous_user(self):
        request = self.factory.get('/fake-path')
        request.user = AnonymousUser()

        response = NoLoginRequiredView.as_view()(request)
        self.assertEqual(response.status_code, 200)

    def test_no_login_required_mixin_user(self):
        request = self.factory.get('/fake-path')
        request.user = self.user

        with self.assertRaises(PermissionDenied):
            NoLoginRequiredView.as_view()(request)

    def test_action_list_mixin(self):
        request = self.factory.get('/fake-path')
        response = ActionListView.as_view()(request)
        self.assertEqual(
            response.context_data['action_list'],
            ActionListView.action_list
        )

    def test_user_create_mixin(self):
        content_type = ContentType.objects.get(
            app_label="auth", model="user"
        )

        request = self.factory.post('/fake-path', {
            'content_type': content_type.id,
            'object_id': self.user.id,
            'object_repr': "Test {}".format(self.user),
            'action_flag': 1,
            'change_message': '',
        })
        request.user = self.user
        response = UserCreateView.as_view()(request)
        self.assertEqual(isinstance(response, LogEntry), True)
        self.assertNotEqual(response, None)

    def test_create_message_mixin(self):
        request = self.factory.post('/fake-path', {
            'username': 'gclooney',
            'email': 'gclooney@test.com',
        })
        request._messages = default_storage(request)
        CreateMessageView.as_view()(request)

        self.assertEqual(len(get_messages(request)), 1)

    def test_update_message_mixin(self):
        request = self.factory.post('/fake-path', {
            'username': 'gclooney',
            'email': 'gclooney@test.com',
        })
        request._messages = default_storage(request)
        UpdateMessageView.as_view()(request)

        self.assertEqual(len(get_messages(request)), 1)

    def test_delete_message_mixin(self):
        request = self.factory.post('/fake-path', {})
        request._messages = default_storage(request)
        DeleteMessageView.as_view()(request)

        self.assertEqual(len(get_messages(request)), 1)

    def test_parent_create_mixin(self):
        content_type = ContentType.objects.get(
            app_label='contenttypes', model='contenttype',
        )

        request = self.factory.post('/fake-path', {
            'codename': 'test_user',
            'name': 'Test user'
        })
        response = ParentCreateView.as_view()(
            request,
            pk_parent=content_type.id,
        )
        self.assertEqual(response.content_type, content_type)

    def test_parent_single_object_mixin(self):
        content_type = ContentType.objects.get(
            app_label='contenttypes', model='contenttype',
        )
        permission, created = Permission.objects.get_or_create(
            content_type=content_type,
            codename='test_user',
            name='Test user'
        )

        request = self.factory.get('/fake-path')
        response = ParentSingleObjectView.as_view()(
            request,
            pk_parent=content_type.id,
            pk=permission.id
        )
        self.assertEqual(response.context_data['object'], permission)

    def test_extra_forms_and_formsets_mixin(self):
        content_type = ContentType.objects.get(
            app_label='contenttypes', model='contenttype',
        )

        request = self.factory.get('/fake-path')
        response = ExtraFormsAndFormsetsView.as_view()(
            request,
            pk=content_type.pk,
        )
        self.assertNotEqual(response.context_data['form'], None)
        self.assertEqual(len(response.context_data['formset_list']), 1)

        request = self.factory.post('/fake-path', {
            'app_label': content_type.app_label,
            'model': content_type.model,
            'permission_set-TOTAL_FORMS': 4,
            'permission_set-INITIAL_FORMS': 3,
            'permission_set-MIN_NUM_FORMS': 0,
            'permission_set-MAX_NUM_FORMS': 1000,
            'permission_set-0-name': "Can add content type",
            'permission_set-0-codename': "add_contenttype",
            'permission_set-0-id': 13,
            'permission_set-0-content_type': 5,
            'permission_set-1-name': "Can change content type",
            'permission_set-1-codename': "change_contenttype",
            'permission_set-1-id': 14,
            'permission_set-1-content_type': 5,
            'permission_set-2-name': "Can delete content type",
            'permission_set-2-codename': "delete_contenttype",
            'permission_set-2-id': 15,
            'permission_set-2-content_type': 5,
            'permission_set-3-name': "Can test content type",
            'permission_set-3-codename': "test_contenttype",
            'permission_set-3-content_type': 5,
        })
        response = ExtraFormsAndFormsetsView.as_view()(
            request,
            pk=content_type.pk,
        )
        self.assertEqual(
            response.permission_set.get(codename='test_contenttype').codename,
            'test_contenttype'
        )

    def test_extra_forms_and_formsets_and_update_message_mixin(self):
        content_type = ContentType.objects.get(
            app_label='contenttypes', model='contenttype',
        )

        request = self.factory.post('/fake-path', {
            'app_label': content_type.app_label,
            'model': content_type.model,
            'permission_set-TOTAL_FORMS': 4,
            'permission_set-INITIAL_FORMS': 3,
            'permission_set-MIN_NUM_FORMS': 0,
            'permission_set-MAX_NUM_FORMS': 1000,
            'permission_set-0-name': "Can add content type",
            'permission_set-0-codename': "add_contenttype",
            'permission_set-0-id': 13,
            'permission_set-0-content_type': 5,
            'permission_set-1-name': "Can change content type",
            'permission_set-1-codename': "change_contenttype",
            'permission_set-1-id': 14,
            'permission_set-1-content_type': 5,
            'permission_set-2-name': "Can delete content type",
            'permission_set-2-codename': "delete_contenttype",
            'permission_set-2-id': 15,
            'permission_set-2-content_type': 5,
            'permission_set-3-name': "Can test content type",
            'permission_set-3-codename': "test_contenttype",
            'permission_set-3-content_type': 5,
        })
        request._messages = default_storage(request)
        response = ExtraFormsAndFormsetsAndUpdateView.as_view()(
            request,
            pk=content_type.pk,
        )
        self.assertEqual(
            response.permission_set.get(codename='test_contenttype').codename,
            'test_contenttype'
        )
        self.assertEqual(len(get_messages(request)), 1)


class MailTest(TestCase):
    def test_send_email_error(self):
        email = SendEmail(
            to='test@test.com',
            subject="This email should raise an error"
        )
        with self.assertRaises(Exception):
            email.send()

    def test_send_email_only_content(self):
        email = SendEmail(
            to='test@test.com',
            subject="This is the subject",
            content="This is the email content."
        )
        self.assertEqual(email.send(), 1)



# -*- coding: utf-8 -*-
from django import template
from django.contrib.admin.utils import NestedObjects
from django.core.urlresolvers import reverse
from django.db import DEFAULT_DB_ALIAS
from django.utils.text import slugify

from ..boilerplate import get_boilerplate_setting

register = template.Library()

URL_LIST_SUFFIX = get_boilerplate_setting('model_url_list_suffix', '_list')
APP_URL_INDEX = get_boilerplate_setting('app_url_index', 'home')


@register.simple_tag
def url_replace(request, field, value):
    dict_ = request.GET.copy()
    dict_[field] = value
    return dict_.urlencode()


@register.filter
def get_deleted_objects(object):
    """
    List the related objects before delete an object
    """
    collector = NestedObjects(using=DEFAULT_DB_ALIAS)
    collector.collect([object])

    return collector.nested()


"""

Form filters

"""


@register.filter
def form_model_name(value):
    """
    Return the model verbose name of a form model
    """
    try:
        return value._meta.model._meta.verbose_name
    except Exception:
        return value.__class__.__name__


@register.filter
def form_model_name_plural(value):
    """
    Return the model verbose name plural of a form model
    """
    return value._meta.model._meta.verbose_name_plural


@register.filter
def form_model_url(value):
    """
    Return the model list url name of a form model
    """
    return (
        value._meta.model._meta.app_label +
        ':' +
        value._meta.model.__name__.lower() +
        URL_LIST_SUFFIX
    )


@register.filter
def form_app_name(value):
    """
    Return the app name of a form model
    """
    return value._meta.model._meta.app_config.verbose_name


@register.filter
def form_app_url(value):
    """
    Return the app home url of a form model
    """
    return value._meta.model._meta.app_label.lower() + ':' + APP_URL_INDEX


@register.filter
def form_prefix(value):
    """
    Return the form name as a prefix
    """
    return slugify(value.__class__.__name__.lower())


@register.filter
def formset_model_name(value):
    """
    Return the model verbose name of a formset
    """
    return value.model._meta.verbose_name


@register.filter
def formset_model_name_plural(value):
    """
    Return the model verbose name plural of a formset
    """
    return value.model._meta.verbose_name_plural


"""

Model filters

"""


@register.simple_tag
def model_action(value, action):
    """
    **Tag name**
    ::
        model_action

    Return the full url name of an action

    **Usage**
    ::
        {% model_action object action %}

    **Example**
    ::
        {% model_action object 'update' %}
    """
    name = (
        value._meta.model._meta.app_label +
        ':' +
        value._meta.model.__name__.lower() +
        "_" +
        action
    )
    return reverse(name, args=(value.pk,))


@register.simple_tag
def model_child_action(value, parent, action):
    """
    **Tag name**
    ::
        model_child_action

    Return the full url name of an action

    **Usage**
    ::
        {% model_child_action object parent action %}

    **Example**
    ::
        {% model_child_action object parent 'update' %}
    """
    name = (
        parent._meta.model._meta.app_label +
        ':' +
        parent._meta.model.__name__.lower() +
        "_" +
        value._meta.model.__name__.lower() +
        "_" +
        action
    )
    return reverse(name, args=(parent.pk, value.pk,))


@register.filter
def model_name(value):
    """
    Return the model verbose name of an object
    """
    return value._meta.verbose_name


@register.filter
def model_name_plural(value):
    """
    Return the model verbose name plural of an object
    """
    return value._meta.verbose_name_plural


@register.filter
def model_app_name(value):
    """
    Return the app verbose name of an object
    """
    return value._meta.app_config.verbose_name


@register.filter
def model_app_url(value):
    """
    Return the app home url of an object
    """
    return (
        value._meta.app_label.lower().replace(" ", "_") +
        ':' +
        APP_URL_INDEX
    )


@register.filter
def model_url(value):
    """
    Return the model list url name of a model
    """
    return (
        value._meta.model._meta.app_label +
        ':' +
        value._meta.model.__name__.lower() +
        URL_LIST_SUFFIX
    )


"""

Queryset filters

"""


@register.filter
def queryset_app_name(value):
    """
    Return the app verbose name of a queryset
    """
    return value.model._meta.app_config.verbose_name


@register.filter
def queryset_app_url(value):
    """
    Return the app home url name of a queryset
    """
    return value.model._meta.app_label.lower() + ':' + APP_URL_INDEX


@register.filter
def queryset_model_name_plural(value):
    """
    Return the app verbose name plural of a queryset
    """
    return value.model._meta.verbose_name_plural


@register.filter
def queryset_model_url(value):
    """
    Return the model list url name of a queryset
    """
    return (
        value.model._meta.app_label.lower() +
        ':' +
        value.model.__name__.lower() +
        URL_LIST_SUFFIX
    )


@register.simple_tag
def queryset_action(value, action):
    """
    **Tag name**
    ::
        queryset_action

    Return the full url name of an action

    **Parameters**:
            :queryset: A valid queryset of objects

            :action: A valid action name. Ex. namespace:model_action

    **Usage**
    ::
        {% queryset_action queryset action %}

    **Example**
    ::
        {% queryset_action queryset 'add' %}
    """
    name = (
        value.model._meta.app_label +
        ':' +
        value.model.__name__.lower() +
        "_" +
        action
    )
    return reverse(name)
